/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_MouseEventBinding_h__
#define mozilla_dom_MouseEventBinding_h__

#include "js/RootingAPI.h"
#include "jsapi.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/EventTarget.h"
#include "mozilla/dom/Nullable.h"
#include "mozilla/dom/workers/bindings/EventTarget.h"
#include "nsIDOMWindow.h"

class JSObject;
class nsDOMMouseEvent;
class nsIDOMWindow;

namespace mozilla {
namespace dom {

class EventTarget;
struct MouseEventInitAtoms;
class NativePropertyHooks;

namespace workers {

class EventTarget;

} // namespace workers

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct MouseEventInit : public MainThreadDictionaryBase
{
  bool mAltKey;
  bool mBubbles;
  uint16_t mButton;
  uint16_t mButtons;
  bool mCancelable;
  int32_t mClientX;
  int32_t mClientY;
  bool mCtrlKey;
  int32_t mDetail;
  bool mMetaKey;
  nsRefPtr<mozilla::dom::EventTarget> mRelatedTarget;
  int32_t mScreenX;
  int32_t mScreenY;
  bool mShiftKey;
  nsRefPtr<nsIDOMWindow> mView;

  inline MouseEventInit()
  {
  }

private:
  MouseEventInit(const MouseEventInit&) MOZ_DELETE;
  void operator=(const MouseEventInit) MOZ_DELETE;

  static bool
  InitIds(JSContext* cx, MouseEventInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);
};
struct MouseEventInitInitializer : public MouseEventInit
{
  MouseEventInitInitializer();
};


namespace MouseEventBinding {

  typedef nsDOMMouseEvent NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];
  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, nsDOMMouseEvent* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, T* aObject)
  {
    return Wrap(aCx, aScope, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Heap<JSObject*>* aProtoAndIfaceArray, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace MouseEventBinding



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MouseEventBinding_h__
